#include <WScom.h>
#include <WSCfunctionList.h>
#include <WSCbase.h>

#include <WSCmessageDialog.h>

#include "mainwin.h"

#include "download.h"
#include "memmap.h"
#include "options.h"
#include "region.h"
#include "target.h"
#include "util.h"

#define REQ_CONNECT		0x1
#define REQ_MEM_MAP		0x2
#define REQ_LOCAL_MAC		0x4

static void require_flags(target_context_t *tc, unsigned flags)
{
	/* filter out already present flags */
	flags &= ~tc->flags;

	if (flags & REQ_CONNECT) {
		if (target_connect(tc))
			panic("can't connect to target");
		flags |= REQ_CONNECT;

	} else if (flags & REQ_MEM_MAP) {
		if (opt_memory_map_path) {
			mem_map_read_from_file(tc, opt_memory_map_path);
		} else {
			require_flags(tc, REQ_CONNECT);
			target_write_command(tc, "memmap");
			mem_map_read(tc);
		}
		flags |= REQ_MEM_MAP;

	} else if (flags & REQ_LOCAL_MAC) {
		const char zeroes [6] = { 0, };
		assert(memcmp(tc->local_mac, zeroes, 6));
		flags |= REQ_LOCAL_MAC;
	}

	/* XXX need to handle more than one request! */
	tc->flags |= flags;
}

//----------------------------------------------------------
//Function for the event procedure
//----------------------------------------------------------
void btnDownload_act(WSCbase* object){
  //do something...
	WSCmessageDialog* msg = WSGIappMessageDialog ();
	msg->setProperty (WSNwidth,300);
	msg->setProperty (WSNheight,200);
	msg->setProperty (WSNdefaultPosition,True);
	
	command_t *cmd = new_command ();
	char *p;
	
	WSCstring port = cmbPort->getProperty (WSNlabelString);
	WSCstring inputfile = ifdInputFile->getProperty (WSNlabelString);
	WSCstring address = ifdAddress->getProperty (WSNlabelString);
	if (!port.getChars () || !inputfile.getChars () || !address.getChars ()) {
		free (cmd);
		return;
	}
	opt_port = port;
	cmd->input_path = inputfile;
	cmd->addr = strtoul (address, &p, 0);
	if (*p != '\0') {
		msg->setProperty (WSNlabelString, "Invalid address.");
		msg->popup ();
		free (cmd);
		return;
	}
	cmd->info.download.have_address = 1;
	opt_force_locked = togForceLocked->getStatus ();
	opt_verbose = togVerbose->getStatus ();
	
	if (opt_verbose) {
		print_command (cmd);
	}
	
	target_context_t *tc;
	if (!(tc = target_open (opt_port, opt_ethernet ? opt_netif : NULL))) {
		msg->setProperty (WSNlabelString, "Couldn't open connection to target.");
		msg->popup ();
		free (cmd);
		return;
	}
	
	mem_region_t *mr = slurp_file_or_die (cmd->input_path);
	require_flags (tc, REQ_MEM_MAP);
	download_to_addr (tc, mr, cmd->addr);
	
	region_destroy (mr);
	target_close (tc);
	free (cmd);
}
static WSCfunctionRegister  op("btnDownload_act",(void*)btnDownload_act);
